<?php
/**
 * @copyright	Copyright (c) 2013 Skyline Software (http://extstore.com). All rights reserved.
 * @license		GNU General Public License version 2 or later; see LICENSE.txt
 */

// no direct access
defined('_JEXEC') or die();

jimport('joomla.plugin.plugin');
jimport('joomla.application.categories');

/**
 * System - Content Advertisement Plugin
 *
 * @package		Joomla.Plugin
 * @subpakage	Skyline.ContentAds
 */
class plgSystemSL_ContentAds extends JPlugin {
	public $pluginNameHumanReadable;
	public $pluginName = 'sl_contentads';

	/**
	 * Constructor.
	 *
	 * @param 	$subject
	 * @param	array $config
	 */
	function __construct(&$subject, $config = array()) {
		// call parent constructor
		parent::__construct($subject, $config);

		$this->loadLanguage();
		$this->pluginNameHumanReadable	= JText::_('PLG_SYSTEM_SL_CONTENTADS_NAME');
	}


	/**
	 * onContentPrepareForm hook.
	 */
	function onContentPrepareForm($form, $data) {
		// check whether form is valid
		if (!($form instanceof JForm)) {
			$this->_subject->setError('JERROR_NOT_A_FORM');
			return false;
		}

		JForm::addFormPath(dirname(__FILE__));

		if ($form->getName() == 'com_content.article') {
			$form->loadFile($this->_name, true, 'content');
		} else if ($form->getName() == 'com_categories.categorycom_content') {
			JForm::addFormPath(dirname(__FILE__));
			$form->loadFile($this->_name, true, 'category');
		}

		return true;
	}

	/**
	 * onRenderAdminForm hook for K2.
	 */
	function onRenderAdminForm($item, $type, $tab = '') {
		// Load the K2 plugin API
		JLoader::register('K2Parameter', JPATH_ADMINISTRATOR . '/components/com_k2/lib/k2parameter.php');

		$manifest = dirname(__FILE__) . '/' . $this->_name . '.xml';

		if (!empty ($tab)) {
			$path = $type . '-' . $tab;
		} else {
			$path = $type;
		}

		$form	= JForm::getInstance('plg_k2_' . $this->pluginName . '_' . $path, $manifest, array(), true, 'fields[@group="' . $path . '"]');
		$values	= array();

		if ($item->plugins) {
			foreach (json_decode($item->plugins) as $name => $value) {
				$count = 1;
				$values[str_replace($this->pluginName, '', $name, $count)] = $value;
			}

			$form->bind($values);
		}

		$fields = '';

		foreach ($form->getFieldset() as $field) {
			$search		= 'name="' . $field->name . '"';
			$replace	= 'name="plugins[' . $this->pluginName . $field->name . ']"';
			$input		= JString::str_ireplace($search, $replace, $field->__get('input'));
			$fields		.= $field->__get('label') . ' ' . $input;
		}

		if ($fields) {
			$plugin = new JObject;
			$plugin->set('name', $this->pluginNameHumanReadable);
			$plugin->set('fields', $fields);
			return $plugin;
		}
	}

	///////////////////////////////////////////////////////////////////////////

	/**
	 * Common method for process content by event.
	 */
	private function _processContent($event, $context, $article, $params, $limitstart) {
		if ($this->_checkModule()) {
			return;
		}

		$input	= JFactory::getApplication()->input;

		// check if it is possible to insert ad into article
		$params = !empty($params) ? $params : new JRegistry($article->params);

		if (!isset($article->params) || $input->getInt('id') != $article->id || !$params->get('enable', 1)) {
			return '';
		}

		// 1st stage - Check if ad content is set in article
		if ($params->get($event)) {
			return $params->get($event);
		}

		// 2nd stage - Check if ad content is set in category or parent category
		$maxDepth	= $this->params->get('max_depth', 1);

		if (isset($article->catid)) {
			$options	= explode('.', $context);
			$option		= $options[0];
			$extension	= substr($option, strpos($option, '_') + 1);

			$categories = JCategories::getInstance($extension);
			if (!$categories) {
				return false;
			}

			$category	= $categories->get($article->catid);
			$maxDepth	= $this->params->get('max_depth', 1);
			$depth		= 0;

			while ($category && $depth < $maxDepth) {
				$depth++;

				if ($category->getParams()->get($event)) {
					return $category->getParams()->get($event);
				} else {
					$category	= $category->getParent();
				}
			}
		}

		// 3rd state - Check if ad content is set in plugin configuration
		if ($this->params->get($event)) {
			return $this->params->get($event);
		}

		return '';
	}

	/**
	 * onContentAfterTitle hook.
	 */
	public function onContentAfterTitle($context, $article, $params, $limitstart = 0) {
		return $this->_processContent('adAfterDisplayTitle', $context, $article, $params, $limitstart);
	}

	/**
	 * onContentBeforeDisplay hook.
	 */
	public function onContentBeforeDisplay($context, $article, $params, $limitstart = 0) {
		return $this->_processContent('adBeforeDisplayContent', $context, $article, $params, $limitstart);
	}

	/**
	 * onContentAfterDisplay hook.
	 */
	public function onContentAfterDisplay($context, $article, $params, $limitstart = 0) {
		return $this->_processContent('adAfterDisplayContent', $context, $article, $params, $limitstart);
	}

	///////////////////////////////////////////////////////////////////////////

	/**
	 * Method to load K2 category by id
	 */
	private function _loadK2Category($id) {
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);

		$query->select('*')
			->from('#__k2_categories')
			->where('id = ' . (int) $id)
		;
		$db->setQuery($query);

		$item	= $db->loadObject();

		return $item;
	}

	/**
	 * Common method for process k2 content by event.
	 */
	private function _processK2Content($event, $article) {
		$input	= JFactory::getApplication()->input;

		if ($this->_checkModule()) {
			return;
		}

		// Load the K2 plugin API
		JLoader::register('K2Parameter', JPATH_ADMINISTRATOR . '/components/com_k2/lib/k2parameter.php');
		$plugins = new K2Parameter($article->plugins, '', $this->pluginName);

		// check if it is possible to insert ad into article
		if ($input->getString('view') != 'item' || $input->getInt('id') != $article->id || !$plugins->get('enable', 1)) {
			return '';
		}

		// 1st stage - Check if ad content is set in article
		if ($plugins->get($event)) {
			return $plugins->get($event);
		}

		// 2nd stage - Check if ad content is set in category or parent category
		$category	= $this->_loadK2Category($article->catid);
		$maxDepth	= $this->params->get('max_depth', 1);
		$depth		= 0;

		while ($category && $depth < $maxDepth) {
			$depth++;
			$cparams	= new K2Parameter($category->plugins, '', $this->pluginName);
			if ($cparams->get($event)) {
				return $cparams->get($event);
			} else {
				$category	= $this->_loadK2Category($category->parent);
			}

		}

		// 3rd state - Check if ad content is set in plugin configuration
		if ($this->params->get($event)) {
			return $this->params->get($event);
		}
	}

	/**
	 * onK2BeforeDisplay hook.
	 */
	function onK2BeforeDisplay($item, $params, $limitstart) {
		return $this->_processK2Content('adBeforeDisplay', $item);
	}

	/**
	 * onK2BeforeDisplayContent hook.
	 */
	function onK2BeforeDisplayContent($item, $params, $limitstart) {
		return $this->_processK2Content('adBeforeDisplayContent', $item);
	}

	/**
	 * onK2AfterDisplayTitle hook.
	 */
	function onK2AfterDisplayTitle($item, $params, $limitstart) {
		return $this->_processK2Content('adAfterDisplayTitle', $item);
	}

	/**
	 * onK2AfterDisplayContent hook.
	 */
	function onK2AfterDisplayContent($item, $params, $limitstart) {
		return $this->_processK2Content('adAfterDisplayContent', $item);
	}

	/**
	 * onK2AfterDisplay hook.
	 */
	function onK2AfterDisplay($item, $params, $limitstart) {
		return $this->_processK2Content('adAfterDisplay', $item);
	}

	function onBeforeRenderModule() {
		global $contentAds;

		$contentAds = false;
	}

	function onAfterRenderModule() {
		global $contentAds;

		$contentAds	= true;
	}

	private function _checkModule() {
		$backtrack = debug_backtrace();

		foreach ($backtrack as $track) {
			if (isset($track['file']) && strpos($track['file'], 'module.php')) {
				return true;
			}
		}

		return false;
	}
}